# Copyright 2015 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

from datetime import datetime

import sqlite3
import uuid

from rutabaga.config import RutabagaConfig


CONFIG = RutabagaConfig()


def _db_conn():
    return sqlite3.connect(CONFIG.get('database'))


def _execute(query, bindings=[], conn=None, rows=None):
    if not conn:
        conn = _db_conn()
    with conn:
        conn.row_factory = sqlite3.Row
        cursor = conn.cursor()
        return (cursor.execute(query, bindings).fetchone(), cursor.rowcount)


def create_token(username, conn=None):
    """Create a unique token, or return existing valid token.

    'username secret' are effectively 'username password' for
    proxy basic auth.
    """
    unique_id = uuid.uuid4().hex
    timestamp = datetime.utcnow().isoformat()
    return {'username': username,
            'secret': unique_id,
            'timestamp': timestamp}


def store_token(token, conn=None):
    """Persist token."""
    if retrieve_token(token['username'], token['secret'], conn=conn):
        return  # token exists
    query = ("INSERT INTO Token (secret, username, timestamp) "
             "values (:secret, :username, :timestamp);")
    _execute(query, token, conn=conn)


def retrieve_token(username, secret, conn=None):
    """Retrieve a valid token from database by secret."""
    query = ("SELECT secret, username, timestamp, valid "
             "FROM Token WHERE secret=:secret AND username=:username "
             "AND valid=1 AND datetime(timestamp, '+{ttl} minute') "
             ">= datetime('now');").format(ttl=CONFIG.get('token_ttl_minutes'))

    (token, _) = _execute(query, [secret, username], conn=conn)
    if token:
        return dict(token)
    else:
        return None


def invalidate_token(username, conn=None):
    """Invalidate a token."""
    query = "UPDATE Token SET valid=0 WHERE username=:username;"
    (_, count) = _execute(query, [username], conn=conn)
    return bool(count)


def delete_invalid_tokens(conn=None):
    """Delete invalid tokens.

    Called from sweeper.
    """
    query = ("DELETE FROM Token WHERE valid=0 OR "
             "datetime(timestamp, '+{ttl} minute') "
             "<= datetime('now');").format(ttl=CONFIG.get('token_ttl_minutes'))
    (_, count) = _execute(query, conn=conn)
    return count
