# Copyright 2015 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

from __future__ import print_function

from datetime import timedelta
import sqlite3
import unittest
import uuid

import iso8601
from testtools import TestCase

from rutabaga import auth
from rutabaga.config import RutabagaConfig
from rutabaga.helpers import validate_uuid4


class AuthTestCase(TestCase):

    def _db_conn(self):
        conn = sqlite3.connect(':memory:')
        with open('db/rutabaga.schema', 'r') as schema:
            conn.executescript(schema.read())
            conn.commit()
        return conn

    def _username(self):
        return 'BUILD-%s' % uuid.uuid4().hex

    def _create_expired_token(self):
        # Create a token with an expired timestamp.
        token = self._create_valid_token()
        minutes = 1 + self.config.get('token_ttl_minutes')
        expired_delta = timedelta(minutes=-minutes)
        timestamp = iso8601.parse_date(token['timestamp'])
        token['timestamp'] = str(timestamp + expired_delta)
        return token

    def _create_valid_token(self):
        return auth.create_token(
            username=self._username(),
            conn=self.db)

    def _row_count(self, conn):
        cursor = conn.cursor()
        cursor.execute("SELECT Count(*) FROM Token")
        return cursor.fetchone()[0]

    def setUp(self):
        super(AuthTestCase, self).setUp()
        self.config = RutabagaConfig()
        self.db = self._db_conn()
        self.token = self._create_valid_token()
        self.expired_token = self._create_expired_token()

    def tearDown(self):
        super(AuthTestCase, self).tearDown()
        self.db.close()

    def test_create_token(self):
        self.assertTrue(validate_uuid4(self.token['secret']))
        self.assertTrue(iso8601.parse_date(self.token['timestamp']))

    def test_store_token(self):
        with self.db as conn:
            auth.store_token(self.token, conn=conn)
            cursor = conn.cursor()
            cursor.execute("SELECT secret FROM Token")
        self.assertEqual(self.token['secret'], cursor.fetchone()[0])

    def test_retrieve_token(self):
        with self.db as conn:
            auth.store_token(self.token, conn=conn)
        token = auth.retrieve_token(
            self.token['username'], self.token['secret'], conn=conn)
        self.assertEqual(self.token['username'], token['username'])

    def test_invalidate_invalid_token(self):
        with self.db as conn:
            self.assertFalse(auth.invalidate_token('junk', conn=conn))

    def test_invalidate_token(self):
        with self.db as conn:
            auth.store_token(self.token, conn=conn)
        self.assertTrue(
            auth.invalidate_token(self.token['username'], conn=conn))
        self.assertIsNone(auth.retrieve_token(
            self.token['username'], self.token['secret'], conn=conn))

    def test_delete_expired_tokens(self):
        with self.db as conn:
            auth.store_token(self.token, conn=conn)  # valid
            for _ in range(3):
                expired_token = self._create_expired_token()
                auth.store_token(expired_token, conn=conn)
            auth.delete_invalid_tokens(conn=conn)
        self.assertIsNot(None, auth.retrieve_token(
            self.token['username'], self.token['secret'], conn=conn))
        self.assertIsNone(auth.retrieve_token(
            self.expired_token['username'],
            self.expired_token['secret'],
            conn=conn))


if __name__ == '__main__':
    unittest.main()
