# Copyright 2015 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

from datetime import datetime
import os
import sqlite3
import sys
import unittest

from fixtures import EnvironmentVariable
from testtools import TestCase
from webtest import TestApp

import rutabaga
from rutabaga.config import RutabagaConfig


class ApiBaseTestCase(TestCase):

    tempdb = 'tempdb'
    config = RutabagaConfig()

    def _db_conn(self):
        conn = sqlite3.connect(self.tempdb)
        with open('db/rutabaga.schema', 'r') as schema:
            conn.executescript(schema.read())
            conn.commit()
        return conn

    def create_token(self):
        self.username = 'BUILD-9876-{}'.format(datetime.utcnow().timestamp())
        # Create a token authenticated as admin
        self.app.authorization = ('Basic', (self.admin_username,
                                            self.admin_secret))
        return self.app.post_json('/tokens', {'username': self.username}).json

    def setUp(self):
        super(ApiBaseTestCase, self).setUp()
        self.useFixture(EnvironmentVariable("DATABASE", self.tempdb))
        self.app = TestApp(rutabaga.main({}))
        self.db = self._db_conn()
        self.admin_secret = self.config.get('admin_api_secret')
        self.username = 'BUILD-9876-{}'.format(datetime.utcnow().timestamp())
        self.admin_username = '{prefix}{username}'.format(
            prefix=self.config.get('admin_api_username_prefix'),
            username=self.config.get('admin_api_username'))

    def tearDown(self):
        super(ApiBaseTestCase, self).tearDown()
        self.db.close()
        os.remove(self.tempdb)


class ApiRootTestCase(ApiBaseTestCase):

    def setUp(self):
        super(ApiRootTestCase, self).setUp()
        try:
            from rutabaga import version_info
            self.revision_id = version_info.REVISION_ID
        except ImportError:
            print('\nNo version_info, run: `make`')

    def test_root_endpoint_response(self):
        root_resp = self.app.get('/')
        self.assertIn('Rutabaga API Root', root_resp)

    def test_version_info(self):
        root_resp = self.app.get('/')
        self.assertIn(self.revision_id,
                      root_resp.headers['X-Rutabaga-Revision'])


class ApiTokenTestCase(ApiBaseTestCase):

    # Create
    def test_create_token_no_auth(self):
        self.app.authorization = None
        resp = self.app.post_json('/tokens', {'username': self.username},
                                  expect_errors=True)
        self.assertEqual(401, resp.status_code)

    def test_create_token_token_auth(self):
        # Only an authenticated admin can create tokens
        token = self.create_token()
        self.app.authorization = ('Basic', (token['username'],
                                            token['secret']))
        resp = self.app.post_json('/tokens', {'username': self.username},
                                  expect_errors=True)
        self.assertEqual(401, resp.status_code)

    def test_create_token(self):
        token = self.create_token()
        self.assertIn(self.username, token['username'])

    def test_create_token_reused_username_invalid(self):
        self.create_token()
        resp = self.app.post_json('/tokens', {'username': self.username},
                                  expect_errors=True)
        self.assertEqual(409, resp.status_code)

    def test_invalid_post(self):
        self.app.authorization = ('Basic', (self.admin_username,
                                            self.admin_secret))
        resp = self.app.post_json('/tokens', {'junk': 'junk'},
                                  expect_errors=True)
        self.assertIn('username is missing',
                      resp.json['errors'][0]['description'])
        self.assertEqual(400, resp.status_code)

    def test_create_token_with_reserved_admin_prefix(self):
        # This _should_ be # BUILD-1234 etc.
        self.app.authorization = ('Basic', (self.admin_username,
                                            self.admin_secret))
        admin_prefix = self.config.get('admin_api_username_prefix')
        admin_username = '{}h4x'.format(admin_prefix)
        resp = self.app.post_json('/tokens', {'username': admin_username},
                                  expect_errors=True)
        self.assertEqual(400, resp.status_code)
        self.assertIn('invalid username',
                      resp.json_body['errors'][0]['description'])

    # Delete / Invalidate
    def test_invalidate_bad_username(self):
        resp = self.app.delete('/tokens/junk', expect_errors=True)
        self.assertEqual(401, resp.status_code)

    def test_invalidate_no_auth(self):
        token = self.create_token()
        self.app.authorization = None
        resp = self.app.delete(
            '/tokens/{}'.format(token['username']),
            expect_errors=True)
        self.assertEqual(401, resp.status_code)

    def test_invalidate_admin_auth(self):
        # Ensure admin can invalidate any token.
        token = self.create_token()
        resp = self.app.delete('/tokens/{}'.format(token['username']))
        self.assertEqual(200, resp.status_code)

    def test_invalidate_token_auth(self):
        # Ensure non-admin can only invalidate same token.
        token = self.create_token()
        token2 = self.create_token()
        self.app.authorization = ('Basic',
                                  (token['username'], token['secret']))
        resp = self.app.delete('/tokens/{}'.format(token2['username']),
                               expect_errors=True)
        self.assertEqual(401, resp.status_code)
        resp = self.app.delete('/tokens/{}'.format(token['username']),
                               expect_errors=True)
        self.assertEqual(200, resp.status_code)


class ApiTokenValidationTestCase(ApiBaseTestCase):

    def test_invalid_validate(self):
        resp = self.app.post_json(
            '/validate', {'junk': 'junk'}, expect_errors=True)
        self.assertEqual(400, resp.status_code)

    def test_validate_token(self):
        token = self.create_token()
        resp = self.app.post_json(
            '/validate', {'username': token['username'],
                          'secret': token['secret']})
        self.assertEqual(200, resp.status_code)

    def test_invalid_username(self):
        token = self.create_token()
        resp = self.app.post_json(
            '/validate', {'username': 'BUILD-INVALID',
                          'secret': token['secret']},
            expect_errors=True)
        self.assertEqual(404, resp.status_code)

    def test_invalid_secret(self):
        token = self.create_token()
        resp = self.app.post_json(
            '/validate', {'username': token['username'],
                          'secret': 'junk'},
            expect_errors=True)
        self.assertEqual(404, resp.status_code)


if __name__ == '__main__':
    unittest.main()
