# Copyright 2015 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import sqlite3

from cornice.resource import (
    resource,
    view,
    )
from cornice.util import extract_json_data
from pyramid.authentication import BasicAuthAuthenticationPolicy as AuthPolicy
import pyramid.httpexceptions as exc
from pyramid.response import Response

from rutabaga import auth
try:
    from rutabaga import version_info
except ImportError:
    version_info = None
from rutabaga.config import RutabagaConfig


@resource(path='/')
class RootAPI(object):

    def __init__(self, request):
        super(RootAPI, self).__init__()
        self.request = request
        self.config = RutabagaConfig()

    def get(self):
        """Provide a simple text response with the X-Rutabaga-Revision header
        set to the current revision_id.
        """
        text_response = "Rutabaga API Root"
        if version_info:
            revision_id = version_info.REVISION_ID
            return Response(text_response,
                            headers={'X-Rutabaga-Revision': revision_id})
        else:
            return text_response


@resource(collection_path='/tokens', path='/tokens/{username}')
class TokenAPI(object):

    def __init__(self, request):
        super(TokenAPI, self).__init__()
        self.request = request
        self.config = RutabagaConfig()

    def validate_create_token(self, request):
        if 'admin' not in request.effective_principals:
            raise exc.HTTPUnauthorized()
        admin_prefix = self.config.get('admin_api_username_prefix')
        if 'username' not in request.json_body:
            request.errors.add('body', 'username', 'username is missing.')
        elif request.json_body['username'].startswith(admin_prefix):
            request.errors.add('body', 'username', 'invalid username.')

    def validate_delete_token(self, request):
        if 'admin' in request.effective_principals:
            return request
        credentials = AuthPolicy._get_credentials(self, request)
        username = request.matchdict['username']
        if credentials:
            _, auth_secret = credentials
            token = auth.retrieve_token(username, auth_secret)
            if not token:
                raise exc.HTTPUnauthorized()
        else:
            raise exc.HTTPUnauthorized()
        return request

    @view(validators=('validate_create_token'))
    def collection_post(self):
        """Get a new time limited auth token.

        :param username: a unique identifier for this token.

        Returns:
          A json object in the form:
          {'username': username, 'secret': unique_id, 'timestamp': timestamp}
        """
        username = extract_json_data(self.request).get('username')
        try:
            token = auth.create_token(username=username)
            auth.store_token(token)
        except sqlite3.IntegrityError:
            raise exc.HTTPConflict()
        return token

    @view(validators=('validate_delete_token'))
    def delete(self):
        """Invalidate an existing token.

        :param secret: auth token id.
        """
        username = self.request.matchdict['username']
        token = auth.invalidate_token(username)
        if not token:
            return exc.HTTPNotFound()
        return exc.HTTPOk()


@resource(path='/validate', validators=('validate_validation_req'))
class TokenValidationAPI(object):

    def __init__(self, request):
        super(TokenValidationAPI, self).__init__()
        self.request = request

    def validate_validation_req(self, request):
        if 'username' not in request.json_body:
            request.errors.add('body', 'username', 'username is missing.')

        if 'secret' not in request.json_body:
            request.errors.add('body', 'secret', 'secret is missing.')
        return request

    def post(self):
        """Validate a token (username and secret pair).

        :param username: username for existing token.
        :param secret: secret of existing token.
        """
        username = extract_json_data(self.request).get('username')
        secret = extract_json_data(self.request).get('secret')
        token = auth.retrieve_token(username, secret)
        if not token:
            return exc.HTTPNotFound()
        return {'username': token['username']}
